@@ -61,7 +61,7 @@ except ImportError:
         return _convert(node_or_string)
 
 
-__version__ = '1.2.2'
+__version__ = '1.3'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service',
            'format_exception', 'read_config']
 
@@ -139,6 +139,8 @@ _methods_6_1 = {
 #  - (not in 6.1) 'object': ['obj_list']
 #  - 'common': ['get_available_updates', 'get_migration_scripts',
 #               'set_loglevel']
+_cause_message = ("\nThe above exception was the direct cause "
+                  "of the following exception:\n\n")
 
 
 def mixedcase(s, _cache={}):
@@ -174,17 +176,25 @@ def format_exception(exc_type, exc, tb, limit=None, chain=True,
 
     This wrapper is a replacement of ``traceback.format_exception``
     which formats the error and traceback received by XML-RPC.
+    If `chain` is True, then the original exception is printed too.
     """
-    # Format readable 'Fault' errors
+    values = _format_exception(exc_type, exc, tb, limit=limit)
     if ((issubclass(exc_type, Fault) and
          isinstance(exc.faultCode, basestring))):
+        # Format readable 'Fault' errors
         etype, _, msg = exc.faultCode.partition('--')
+        server_tb = None
         if etype.strip() != 'warning':
             msg = exc.faultCode
             if not msg.startswith('FATAL:'):
-                msg += '\n' + exc.faultString
-        return ['%s: %s\n' % (exc_type.__name__, msg.strip())]
-    return _format_exception(exc_type, exc, tb, limit=limit)
+                server_tb = exc.faultString
+        fault = '%s: %s\n' % (exc_type.__name__, msg.strip())
+        if chain:
+            values = [server_tb or fault, _cause_message] + values
+            values[-1] = fault
+        else:
+            values = [server_tb or fault]
+    return values
 
 
 def read_config(section=None):
@@ -216,9 +226,10 @@ def issearchdomain(arg):
       - ``[('name', '=', 'mushroom'), ('state', '!=', 'draft')]``
       - ``['name = mushroom', 'state != draft']``
       - ``[]``
-      - ``'state != draft'``
-      - ``('state', '!=', 'draft')``
     """
+    # These ones are supported but discouraged:
+    # - 'state != draft'
+    # - ('state', '!=', 'draft')
     return isinstance(arg, (list, tuple, basestring)) and not (arg and (
         # Not a list of ids: [1, 2, 3]
         isinstance(arg[0], int_types) or
@@ -542,8 +553,14 @@ class Client(object):
                 search_params = searchargs(params[:1], kwargs, context)
                 ordered = len(search_params) > 3 and search_params[3]
                 ids = self._execute(obj, 'search', *search_params)
+            elif isinstance(params[0], list):
+                ordered = kwargs.pop('order', False) and params[0]
+                ids = set(params[0])
+                ids.discard(False)
+                if not ids:
+                    return [False] * len(ordered)
+                ids = sorted(ids)
             else:
-                ordered = kwargs.pop('order', False)
                 ids = params[0]
             if not ids:
                 return []
@@ -574,7 +591,10 @@ class Client(object):
             # when received from the server
             assert len(res) == len(set(ids))
             resdic = dict([(val['id'], val) for val in res])
-            res = [resdic[id_] for id_ in ids]
+            if isinstance(ordered, list):
+                res = [id_ and resdic[id_] for id_ in ordered]
+            else:
+                res = [resdic[id_] for id_ in ids]
         return res
 
     def exec_workflow(self, obj, signal, obj_id):
@@ -712,11 +732,11 @@ class Client(object):
             return res
         if fmt:
             if isinstance(res, list):
-                return [fmt % d for d in res]
+                return [(d and fmt % d) for d in res]
             return fmt % res
         if fmt == ():
             if isinstance(res, list):
-                return [d[fields[0]] for d in res]
+                return [(d and d[fields[0]]) for d in res]
             return res[fields[0]]
         return res
 
@@ -904,8 +924,22 @@ class Model(object):
         else:
             assert not params and not kwargs
         return RecordList(self, domain, context=context)
-    # undocumented alias
-    get = browse
+
+    def get(self, domain, context=None):
+        """Return a single :class:`Record`.
+
+        The argument `domain` accepts a single integer ``id`` or a
+        search domain.  The return value is a :class:`Record` or None.
+        If multiple records are found, a ``ValueError`` is raised.
+        """
+        if isinstance(domain, int_types):
+            return Record(self, domain, context=context)
+        assert issearchdomain(domain)
+        params = searchargs((domain,), context=context)
+        ids = self._execute('search', *params)
+        if len(ids) > 1:
+            raise ValueError('domain matches too many records (%d)' % len(ids))
+        return ids and Record(self, ids[0], context=context) or None
 
     def create(self, values, context=None):
         """Create a :class:`Record`.
@@ -925,27 +959,23 @@ class Model(object):
         the value is wrapped in a Record or a RecordList.
         Return a dictionary with the same keys as the `values` argument.
         """
-        get_model = self.client.model
-        new_values = {}
         for key, value in values.items():
             if key == 'id':
-                new_values['id'] = value
                 continue
             field = self._fields[key]
             field_type = field['type']
-            if not value:
-                pass
-            elif field_type == 'many2one':
-                rel_model = get_model(field['relation'])
-                value = Record(rel_model, value, context=context)
+            if field_type == 'many2one':
+                if value:
+                    rel_model = self.client.model(field['relation'])
+                    values[key] = Record(rel_model, value, context=context)
             elif field_type in ('one2many', 'many2many'):
-                rel_model = get_model(field['relation'])
-                value = RecordList(rel_model, value, context=context)
+                rel_model = self.client.model(field['relation'])
+                values[key] = RecordList(rel_model, value, context=context)
             elif field_type == 'reference':
-                res_model, res_id = value.split(',')
-                value = Record(get_model(res_model), int(res_id))
-            new_values[key] = value
-        return new_values
+                if value:
+                    res_model, res_id = value.split(',')
+                    values[key] = Record(self.client.model(res_model), int(res_id))
+        return values
 
     def _unbrowse_values(self, values):
         """Unwrap the id of Record and RecordList."""
@@ -984,10 +1014,12 @@ class RecordList(object):
     """A sequence of OpenERP :class:`Record`."""
 
     def __init__(self, res_model, ids, context=None):
-        if ids and isinstance(ids[0], list):
-            _ids = [id_ for (id_, name) in ids]
-        else:
-            _ids = ids
+        _ids = []
+        for id_ in ids:
+            if isinstance(id_, list):
+                _ids.append(id_[0])
+            else:
+                _ids.append(id_)
         # Bypass the __setattr__ method
         self.__dict__.update({
             # 'client': res_model.client,
@@ -1029,7 +1061,6 @@ class RecordList(object):
 
         if isinstance(fields, basestring):
             field = self._model._fields.get(fields)
-
             if field:
                 if field['type'] == 'many2one':
                     rel_model = client.model(field['relation'])
@@ -1037,6 +1068,15 @@ class RecordList(object):
                 if field['type'] in ('one2many', 'many2many'):
                     rel_model = client.model(field['relation'])
                     return [RecordList(rel_model, v) for v in values]
+                if field['type'] == 'reference':
+                    records = []
+                    for value in values:
+                        if value:
+                            res_model, res_id = value.split(',')
+                            rel_model = client.model(res_model)
+                            value = Record(rel_model, int(res_id))
+                        records.append(value)
+                    return records
         return values
 
     def write(self, values, context=None):
@@ -1048,8 +1088,11 @@ class RecordList(object):
         return rv
 
     def __getitem__(self, key):
+        idname = self._idnames[key]
+        if idname is False:
+            return False
         cls = isinstance(key, slice) and RecordList or Record
-        return cls(self._model, self._idnames[key], context=self._context)
+        return cls(self._model, idname, context=self._context)
 
     def __getattr__(self, attr):
         context = self._context
@@ -1195,8 +1238,12 @@ class Record(object):
         new_id = self._model._execute('copy', self.id, default, context=context)
         return Record(self._model, new_id)
 
+    def _send(self, signal):
+        """Trigger workflow `signal` for this :class:`Record`."""
+        return self._model.exec_workflow(signal, self.id)
+
     def __dir__(self):
-        return ['read', 'write', 'copy', 'unlink',
+        return ['read', 'write', 'copy', 'unlink', '_send',
                 'id', '_context', '_model', '_model_name',
                 '_name', '_keys', '_fields'] + self._model._keys
 
@@ -1289,7 +1336,8 @@ def _interact(use_pprint=True, usage=USAGE):
             except:
                 # Print readable 'Fault' errors
                 # Work around http://bugs.python.org/issue12643
-                msg = ''.join(format_exception(*sys.exc_info()))
+                exc_type, exc, tb = sys.exc_info()
+                msg = ''.join(format_exception(exc_type, exc, tb, chain=False))
                 print(msg.strip())
 
     warnings.simplefilter('always', UserWarning)
